using Interactables.Interobjects.DoorUtils;
using InventorySystem.Items.Keycards;
using LabApi.Events.Arguments.PlayerEvents;
using LabApi.Events.Handlers;
using LabApi.Features;
using LabApi.Features.Console;

namespace KeycardButModern
{
    public class Plugin: LabApi.Loader.Features.Plugins.Plugin
    {
        public override string Name => "KeycardButModern";
        public override string Description => "Ever thought you wanted your keycard implanted in your body? No? Same.";
        public override string Author => "Code002Lover";
        public override Version Version { get; } = new(1, 0, 0);
        public override Version RequiredApiVersion { get; } = new (LabApiProperties.CompiledVersion);

        private void OnInteractingDoor(PlayerInteractingDoorEventArgs ev)
        {
            if (ev.CanOpen)
            {
                Logger.Debug("Door can be opened, no need for implant check");
                return;
            }

            if (ev.Door.IsLocked)
            {
                Logger.Debug("Door has active locks");
                return;
            }
            
            foreach (var playerItem in ev.Player.Items)
            {
                //is keycard?
                if (playerItem.Type > ItemType.KeycardO5) continue;
                if (playerItem.Base is not KeycardItem keycardItem)
                {
                    continue;
                }
                
                if (!ev.Door.Base.CheckPermissions(keycardItem, out _)) continue;
                ev.Door.IsOpened = !ev.Door.IsOpened;
                Logger.Debug("Door can be opened");
                
                return;
            }
        }
        
        private void OnInteractingGenerator(PlayerInteractingGeneratorEventArgs ev)
        {
            if (!ev.IsAllowed)
            {
                return;
            }

            if (ev.Player.CurrentItem?.Base is KeycardItem keycard)
            {
                if (ev.Generator.Base.CheckPermissions(keycard, out _)) return;
            }


            foreach (var playerItem in ev.Player.Items)
            {
                //is keycard?
                if (playerItem.Type > ItemType.KeycardO5) continue;
                if (playerItem.Base is not KeycardItem keycardItem)
                {
                    continue;
                }
                
                if (!ev.Generator.Base.CheckPermissions(keycardItem, out _)) continue;
                ev.Generator.IsOpen = !ev.Generator.IsOpen;
                Logger.Debug("Generator can be opened");
                
                return;
            }
        }
        
        private void OnInteractingLocker(PlayerInteractingLockerEventArgs ev)
        {
            if (!ev.IsAllowed)
            {
                return;
            }

            if (ev.Chamber.Base.RequiredPermissions == DoorPermissionFlags.None)
            {
                return;
            }
            
            if (ev.Player.CurrentItem?.Base is KeycardItem keycard)
            {
                if (ev.Chamber.Base.CheckPermissions(keycard, out _)) return;
            }
            
            foreach (var playerItem in ev.Player.Items)
            {
                //is keycard?
                if (playerItem.Type > ItemType.KeycardO5) continue;
                if (playerItem.Base is not KeycardItem keycardItem)
                {
                    continue;
                }
                
                if (!ev.Chamber.Base.CheckPermissions(keycardItem, out _)) continue;
                ev.Chamber.IsOpen = !ev.Chamber.IsOpen;
                Logger.Debug("Locker can be opened");
                
                return;
            }
        }
        
        private void OnUnlockingWarhead(PlayerUnlockingWarheadButtonEventArgs ev)
        {
            if (ev.IsAllowed)
            {
                return;
            }
            
            foreach (var playerItem in ev.Player.Items)
            {
                //is keycard?
                if (playerItem.Type > ItemType.KeycardO5) continue;
                if (playerItem.Base is not KeycardItem keycardItem)
                {
                    continue;
                }
                
                if (!AlphaWarheadActivationPanel.Instance.CheckPermissions(keycardItem, out _)) continue;
                ev.IsAllowed = true;
                Logger.Debug("Nuke can be unlocked");                
                return;
            }
        }
        
        public override void Enable()
        {
            Logger.Debug("starting...");
            PlayerEvents.InteractingDoor += OnInteractingDoor;
            PlayerEvents.InteractingGenerator += OnInteractingGenerator;
            PlayerEvents.InteractingLocker += OnInteractingLocker;
            PlayerEvents.UnlockingWarheadButton += OnUnlockingWarhead;
        }

        public override void Disable()
        {
            PlayerEvents.InteractingDoor -= OnInteractingDoor;
            PlayerEvents.InteractingGenerator -= OnInteractingGenerator;
            PlayerEvents.InteractingLocker -= OnInteractingLocker;
            PlayerEvents.UnlockingWarheadButton -= OnUnlockingWarhead;
            Logger.Debug("unloading...");
        }
    }
}